//-----------------------------------------------------------------------------
// 1. DRIVER SETUP - create driver object, midi ports and detection information
//-----------------------------------------------------------------------------

// get the api's entry point
var midiremote_api = require('midiremote_api_v1')

// create the device driver main object
var deviceDriver = midiremote_api.makeDeviceDriver('Native Instruments', 'F1', 'Marco Eugster')

// create objects representing the hardware's MIDI ports
var midiInput = deviceDriver.mPorts.makeMidiInput()
var midiOutput = deviceDriver.mPorts.makeMidiOutput()

// define all possible namings the devices MIDI ports could have
// NOTE: Windows and MacOS handle port naming differently
deviceDriver.makeDetectionUnit().detectPortPair(midiInput, midiOutput)
    .expectInputNameEquals('Traktor Kontrol F1 - 1')
    .expectOutputNameEquals('Traktor Kontrol F1 - 1')
    

var current_channel = 'empty'    // holds the name of the current channel, e.g. Audio 03
var dispspledValue = 0.0         // the value the UI shows. This is the target where we want the green color when the controller knob reaches that point
const hysteresis = 0.01          // no flikcering, turn on green when the value is +-0.01 within the displayed target

const ledRed   =  7
const ledAmber = 18
const ledGreen = 34

function turnOnLED(device, ledID, color)
{
    midiOutput.sendMidi(device, [ 0x9C, ledID, color])
    midiOutput.sendMidi(device, [ 0x80, ledID, 0x00])
}
function turnOffLED(device, ledID)
{
    midiOutput.sendMidi(device, [ 0x9C, ledID, 0x00])
    midiOutput.sendMidi(device, [ 0x80, ledID, 0x00])
}


//-----------------------------------------------------------------------------
// 2. SURFACE LAYOUT - create control elements and midi bindings
//-----------------------------------------------------------------------------
var surfaceElements = {}

// create control element representing your hardware's surface
surfaceElements.knob1 = deviceDriver.mSurface.makeKnob(0, 0, 1, 1.5)
surfaceElements.knob2 = deviceDriver.mSurface.makeKnob(1, 0, 1, 1.5)

// bind midi ports to surface elements
surfaceElements.knob1.mSurfaceValue.mMidiBinding
    .setInputPort(midiInput)
    .setOutputPort(midiOutput)
    .bindToControlChange (12, 2) // channel 0, cc 21


// this is a hidden fader, it is required to receive the change channel events
// this event is required to check, if there are bindings. (e.g. folder channels have no bindings)
// this fader is bound to midi controller that does not exist on the CC121
surfaceElements.fader = deviceDriver.mSurface.makeFader(0,0,0,0)
surfaceElements.fader.mSurfaceValue.mMidiBinding
    .setInputPort(midiInput)
    .setOutputPort(midiOutput)
    .bindToControlChange (0, 0xFF) // dummy


//-----------------------------------------------------------------------------
// 3. HOST MAPPING - create mapping pages and host bindings
//-----------------------------------------------------------------------------

// create at least one mapping page
var page = deviceDriver.mMapping.makePage('Example Mixer Page')

// create host accessing objects
var hostSelectedTrackChannel = page.mHostAccess.mTrackSelection.mMixerChannel

// bind surface elements to host accessing object values
page.makeValueBinding(surfaceElements.knob1.mSurfaceValue, hostSelectedTrackChannel.mPreFilter.mGain).setValueTakeOverModePickup()

// this is the hidden fader binding and event to handle the channel switches
page.makeValueBinding(surfaceElements.fader.mSurfaceValue, hostSelectedTrackChannel.mValue.mVolume)
surfaceElements.fader.mSurfaceValue.mOnTitleChange = function (activeDevice, value, units) {
    if (Object.keys(value).length == 0)
    {
        // if the value has no object, we have no binding and therefor we are e.g. on a folder channel in the project windows
        // in this case we set it to 'empty'
        current_channel = 'empty'
        // no binding -> turn off the led
        turnOffLED(activeDevice, 0x30)
    }
    else
    {
        current_channel = value
    }
}

// this is called whenever the controller knob was changed, unfortunately also when the display only changes -> bug?
surfaceElements.knob1.mSurfaceValue.mOnProcessValueChange = function(activeDevice, value) { 

    if (current_channel == 'empty') // when we are e.g. on a folder in the project window, we do nothing
    {
        return
    }

    // compare the controller value with the displayed value
    if ((value > (dispspledValue-hysteresis)) && (value < (dispspledValue+hysteresis)))
    {
        turnOnLED(activeDevice, 0x30, ledGreen)
        return
    }
    if (value < dispspledValue)
    {
        turnOnLED(activeDevice, 0x30, ledAmber)
    }
    else
    {
        turnOnLED(activeDevice, 0x30, ledRed)
    }

}

// -48db...48db -> 0.0 .. 1.0
function normalizeDBStringValue(strValue)
{
    if (strValue.length == 0)
    {
        return 0.0
    }

    // check if it is -oodB
    if (strValue.charAt(1)=='o')
    {
        return 0.0
    }
    // remove unit 'dB'
    var value = strValue.substr(0, strValue.length-2)
    return value.valueOf()/96.0+0.5

}

// this is called when the value in the UI changes
// if a controller knob is in pickup mode, this is called only, when the controller realy picked up the value
// it is also called, when the channel was changed
// it is called before the mOnProcessValueChange function
surfaceElements.knob1.mSurfaceValue.mOnDisplayValueChange = function(activeDevice, value) { 
    dispspledValue = normalizeDBStringValue(value)   
}